@@ -4,15 +4,11 @@
  * Original copyright: 
  * K8 parts Copyright 2002,2003 Andi Kleen, SuSE Labs.
  * Additional K8 decoding and simplification Copyright 2003 Eric Morton, Newisys Inc 
- * K8 threshold counters decoding Copyright 2005 Jacob Shin, AMD Inc.
+ * K8 threshold counters decoding Copyright 2005,2006 Jacob Shin, AMD Inc.
  * 
  * Subject to the GNU General Public License
  */
 
-/* Todo:
-   write perl script to generate additional decoders for the other bits from k8mca.h 
- */
-
 #include <stdio.h>
 #include "mcelog.h"
 #include "k8.h"
@@ -47,7 +43,7 @@ static char *memoryio[] = {
 	"memory", "res.", "i/o", "generic"
 };
 static char *nbextendederr[] = { 
-	"ECC error", 
+	"RAM ECC error", 
 	"CRC error",
 	"Sync error",
 	"Master abort",
@@ -55,7 +51,7 @@ static char *nbextendederr[] = {
 	"GART error",
 	"RMW error",
 	"Watchdog error",
-	"Chipkill ECC error", 
+	"RAM Chipkill ECC error", 
 	"<9>","<10>","<11>","<12>",
 	"<13>","<14>","<15>"
 };
@@ -85,11 +81,14 @@ static char *highbits[32] = {
 	[0] = "err cpu0",
 };
 static char *k8threshold[] = {
-	"MC0_MISC threshold",
-	"MC1_MISC threshold",
-	"MC2_MISC threshold",
-	"MC3_MISC threshold",
-	"MC4_MISC DRAM errors threshold", /* realized on K8 Rev.F */
+	[0 ... K8_MCELOG_THRESHOLD_DRAM_ECC - 1] = "Unknow threshold counter",
+	[K8_MCELOG_THRESHOLD_DRAM_ECC] = "MC4_MISC0 DRAM threshold",
+	[K8_MCELOG_THRESHOLD_LINK] = "MC4_MISC1 Link threshold",
+	[K8_MCELOG_THRESHOLD_L3_CACHE] = "MC4_MISC2 L3 Cache threshold",
+	[K8_MCELOG_THRESHOLD_FBDIMM] = "MC4_MISC3 FBDIMM threshold",
+	[K8_MCELOG_THRESHOLD_FBDIMM + 1 ... 
+	 K8_MCE_THRESHOLD_TOP - K8_MCE_THRESHOLD_BASE - 1] = 
+                "Unknown threshold counter",
 };
 
 
@@ -127,7 +126,7 @@ static void decode_k8_generic_errcode(u64 status)
 	}
 }
 
-static void decode_k8_dc_mc(u64 status)
+static void decode_k8_dc_mc(u64 status, int *err)
 {
 	unsigned short exterrcode = (status >> 16) & 0x0f;
 	unsigned short errcode = status & 0xffff;
@@ -149,7 +148,7 @@ static void decode_k8_dc_mc(u64 status)
 	decode_k8_generic_errcode(status);
 }
 
-static void decode_k8_ic_mc(u64 status)
+static void decode_k8_ic_mc(u64 status, int *err)
 {
 	unsigned short exterrcode = (status >> 16) & 0x0f;
 	unsigned short errcode = status & 0xffff;
@@ -166,7 +165,7 @@ static void decode_k8_ic_mc(u64 status)
 	decode_k8_generic_errcode(status);
 }
 
-static void decode_k8_bu_mc(u64 status)
+static void decode_k8_bu_mc(u64 status, int *err)
 {
 	unsigned short exterrcode = (status >> 16) & 0x0f;
 
@@ -180,12 +179,12 @@ static void decode_k8_bu_mc(u64 status)
 	decode_k8_generic_errcode(status);
 }
 
-static void decode_k8_ls_mc(u64 status)
+static void decode_k8_ls_mc(u64 status, int *err)
 {
 	decode_k8_generic_errcode(status);
 }
 
-static void decode_k8_nb_mc(u64 status)
+static void decode_k8_nb_mc(u64 status, int *err)
 {
 	unsigned short exterrcode = (status >> 16) & 0x0f;
 
@@ -219,7 +218,7 @@ static void decode_k8_threshold(u64 misc)
 		Wprintf("  Threshold error count overflow\n");
 }
 
-typedef void (*decoder_t)(u64); 
+typedef void (*decoder_t)(u64, int *ismemerr); 
 
 static decoder_t decoders[] = { 
 	[0] = decode_k8_dc_mc,
@@ -229,15 +228,17 @@ static decoder_t decoders[] = {
 	[4] = decode_k8_nb_mc,
 }; 
 
-void decode_k8_mc(struct mce *mce)
+int decode_k8_mc(struct mce *mce)
 {
+	int ismemerr = 0;
 	if (mce->bank < NELE(decoders))
-		decoders[mce->bank](mce->status);
-	else if (mce->bank > MCE_THRESHOLD_BASE &&
-		 mce->bank <= MCE_THRESHOLD_DRAM_ECC)
+		decoders[mce->bank](mce->status, &ismemerr);
+	else if (mce->bank >= K8_MCE_THRESHOLD_BASE &&
+		 mce->bank < K8_MCE_THRESHOLD_TOP)
 		decode_k8_threshold(mce->misc);
 	else
 		Wprintf("  no decoder for unknown bank %u\n", mce->bank);
+	return ismemerr;
 }
 
 char *k8_bank_name(int num)
@@ -246,8 +247,9 @@ char *k8_bank_name(int num)
 	char *s = "unknown";
 	if (num < NELE(k8bank))
 		s = k8bank[num];
-	else if (num >= MCE_THRESHOLD_BASE && num <= MCE_THRESHOLD_DRAM_ECC)
-		s = k8threshold[num - MCE_THRESHOLD_BASE];
+	else if (num >= K8_MCE_THRESHOLD_BASE && 
+		 num < K8_MCE_THRESHOLD_TOP)
+		s = k8threshold[num - K8_MCE_THRESHOLD_BASE];
 	sprintf(buf, "%d %s", num, s);
 	return buf;
 }
