@@ -34,8 +34,6 @@ struct mce {
 
 /* Software defined banks */
 #define MCE_EXTENDED_BANK	128
-#define MCE_THRESHOLD_BASE      (MCE_EXTENDED_BANK + 1) /* MCE_AMD */
-#define MCE_THRESHOLD_DRAM_ECC  (MCE_THRESHOLD_BASE + 4)
 
 #define MCI_THRESHOLD_OVER  (1ULL<<48)  /* threshold error count overflow */
 
@@ -61,3 +59,12 @@ struct mce {
 #endif
 
 void Wprintf(char *fmt, ...) PRINTFLIKE;
+void Eprintf(char *fmt, ...) PRINTFLIKE;
+void Lprintf(char *fmt, ...) PRINTFLIKE;
+
+enum cputype {
+	CPU_GENERIC,
+	CPU_CORE2,
+	CPU_K8,
+	CPU_P4
+};
